@@ -16,7 +16,7 @@ GNU Affero General Public License for more details.
 You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
-import base64
+
 import json
 import logging
 import os
@@ -32,22 +32,38 @@ from .client import (
     BackendResponse,
     BulkUploadParams,
     BulkUploadResponse,
-    BulkUploadStatistics,
+    BulkUploadValidateStatusResponse,
     Client,
     DeleteDetectionsParams,
     DeleteDetectionsResponse,
     DeleteSavedQueriesParams,
     DeleteSavedQueriesResponse,
-    ListManagedSchemasResponse,
+    FeatureFlagsParams,
+    FeatureFlagsResponse,
+    GenerateEnrichedEventParams,
+    GenerateEnrichedEventResponse,
+    GetRuleBodyParams,
+    GetRuleBodyResponse,
     ListSchemasParams,
-    ManagedSchema,
-    PantherSDKBulkUploadParams,
-    PantherSDKBulkUploadResponse,
-    UpdateManagedSchemaParams,
-    UpdateManagedSchemaResponse,
+    ListSchemasResponse,
+    MetricsParams,
+    MetricsResponse,
+    PerfTestParams,
+    PermanentBackendError,
+    ReplayResponse,
+    Schema,
+    TestCorrelationRuleParams,
+    TestCorrelationRuleResponse,
+    TranspileFiltersParams,
+    TranspileFiltersResponse,
+    TranspileToPythonParams,
+    TranspileToPythonResponse,
+    UpdateSchemaParams,
+    UpdateSchemaResponse,
     backend_response_failed,
+    to_bulk_upload_response,
 )
-from .errors import is_upload_in_progress_error
+from .errors import is_retryable_error
 
 LAMBDA_CLIENT_NAME = "lambda"
 AWS_PROFILE_ENV_KEY = "AWS_PROFILE"
@@ -57,6 +73,7 @@ def decode_body(res: BackendResponse) -> typing.Any:
     try:
         return json.loads(res.data["body"])
     except json.decoder.JSONDecodeError as decode_error:
+        # pylint: disable=broad-exception-raised
         raise Exception(res.data["body"]) from decode_error
 
 
@@ -92,6 +109,10 @@ class LambdaClient(Client):
     def check(self) -> BackendCheckResponse:
         return BackendCheckResponse(success=True, message="not implemented")
 
+    def async_bulk_upload(self, params: BulkUploadParams) -> BackendResponse[BulkUploadResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("async uploads not supported with lambda client")
+
     def bulk_upload(self, params: BulkUploadParams) -> BackendResponse[BulkUploadResponse]:
         resp = self._parse_response(
             self._lambda_client.invoke(
@@ -110,29 +131,17 @@ class LambdaClient(Client):
         )
 
         if backend_response_failed(resp):
-            err = BackendError(resp.data)
-
-            err.permanent = True
-            if is_upload_in_progress_error(resp.data):
-                err.permanent = False
+            err = PermanentBackendError(resp.data)
+            err.permanent = not is_retryable_error(resp.data)
 
             raise err
 
         body = decode_body(resp)
+        return to_bulk_upload_response(body)
 
-        default_stats = dict(total=0, new=0, modified=0)
-
-        return BackendResponse(
-            status_code=resp.status_code,
-            data=BulkUploadResponse(
-                rules=BulkUploadStatistics(**body.get("rules", default_stats)),
-                queries=BulkUploadStatistics(**body.get("queries", default_stats)),
-                policies=BulkUploadStatistics(**body.get("policies", default_stats)),
-                data_models=BulkUploadStatistics(**body.get("dataModels", default_stats)),
-                lookup_tables=BulkUploadStatistics(**body.get("lookupTables", default_stats)),
-                global_helpers=BulkUploadStatistics(**body.get("globalHelpers", default_stats)),
-            ),
-        )
+    def bulk_validate(self, params: BulkUploadParams) -> BulkUploadValidateStatusResponse:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("bulk validate is not supported with lambda client")
 
     def delete_detections(
         self, params: DeleteDetectionsParams
@@ -200,9 +209,7 @@ class LambdaClient(Client):
             ),
         )
 
-    def list_managed_schemas(
-        self, params: ListSchemasParams
-    ) -> BackendResponse[ListManagedSchemasResponse]:
+    def list_schemas(self, params: ListSchemasParams) -> BackendResponse[ListSchemasResponse]:
         res = self._parse_response(
             self._lambda_client.invoke(
                 FunctionName="panther-logtypes-api",
@@ -220,7 +227,7 @@ class LambdaClient(Client):
         schemas = []
         for result in res.data["results"]:
             schemas.append(
-                ManagedSchema(
+                Schema(
                     created_at=result.get("createdAt", ""),
                     description=result.get("description", ""),
                     is_managed=result.get("isManaged", False),
@@ -229,12 +236,13 @@ class LambdaClient(Client):
                     revision=result.get("revision", ""),
                     spec=result.get("spec", ""),
                     updated_at=result.get("updatedAt", ""),
+                    field_discovery_enabled=result.get("fieldDiscoveryEnabled", False),
                 )
             )
 
-        return BackendResponse(status_code=200, data=ListManagedSchemasResponse(schemas=schemas))
+        return BackendResponse(status_code=200, data=ListSchemasResponse(schemas=schemas))
 
-    def update_managed_schema(self, params: UpdateManagedSchemaParams) -> BackendResponse:
+    def update_schema(self, params: UpdateSchemaParams) -> BackendResponse:
         res = self._parse_response(
             self._lambda_client.invoke(
                 FunctionName="panther-logtypes-api",
@@ -247,6 +255,7 @@ class LambdaClient(Client):
                             "reference_url": params.reference_url,
                             "revision": params.revision,
                             "spec": params.spec,
+                            "fieldDiscoveryEnabled": params.field_discovery_enabled,
                         }
                     }
                 ),
@@ -258,8 +267,8 @@ class LambdaClient(Client):
         schema = res.data.get("result", {})
         return BackendResponse(
             status_code=200,
-            data=UpdateManagedSchemaResponse(
-                schema=ManagedSchema(
+            data=UpdateSchemaResponse(
+                schema=Schema(
                     created_at=schema.get("createdAt", ""),
                     description=schema.get("description", ""),
                     is_managed=schema.get("isManaged", False),
@@ -268,53 +277,39 @@ class LambdaClient(Client):
                     revision=schema.get("revision", ""),
                     spec=schema.get("spec", ""),
                     updated_at=schema.get("updatedAt", ""),
+                    field_discovery_enabled=schema.get("fieldDiscoveryEnabled", False),
                 )
             ),
         )
 
-    def panthersdk_bulk_upload(
-        self, params: PantherSDKBulkUploadParams
-    ) -> BackendResponse[PantherSDKBulkUploadResponse]:
-        resp = self._parse_response(
-            self._lambda_client.invoke(
-                FunctionName="panther-analysis-api",
-                InvocationType="RequestResponse",
-                LogType="None",
-                Payload=self._serialize_request(
-                    {
-                        "sdkUpload": {
-                            "data": base64.b64encode(params.content.encode("utf-8")).decode(
-                                "utf-8"
-                            ),
-                            "userId": self._user_id,
-                        },
-                    }
-                ),
-            )
+    def transpile_simple_detection_to_python(
+        self, params: TranspileToPythonParams
+    ) -> BackendResponse[TranspileToPythonResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException(
+            "transpile simple detections to python is not supported with lambda client"
         )
 
-        if backend_response_failed(resp):
-            err = BackendError(resp.data)
-            err.permanent = True
-            raise err
+    def test_correlation_rule(
+        self, params: TestCorrelationRuleParams
+    ) -> BackendResponse[TestCorrelationRuleResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("test correlation rule is not supported with lambda client")
 
-        body = decode_body(resp)
-        default_stats = dict(total=0, new=0, modified=0)
+    def transpile_filters(
+        self, params: TranspileFiltersParams
+    ) -> BackendResponse[TranspileFiltersResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("transpile filters is not supported with lambda client")
 
-        return BackendResponse(
-            status_code=resp.status_code,
-            data=PantherSDKBulkUploadResponse(
-                rules=BulkUploadStatistics(**body.get("rules", default_stats)),
-                queries=BulkUploadStatistics(**body.get("queries", default_stats)),
-                policies=BulkUploadStatistics(**body.get("policies", default_stats)),
-                data_models=BulkUploadStatistics(**body.get("dataModels", default_stats)),
-            ),
-        )
+    def get_rule_body(self, params: GetRuleBodyParams) -> BackendResponse[GetRuleBodyResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("get rule body is not supported with lambda client")
 
     @staticmethod
     def _serialize_request(data: Dict[str, Any]) -> str:
         logging.debug(">>> %s", data)
-        return json.dumps(data)
+        return json.dumps(data, allow_nan=False)
 
     @staticmethod
     def _parse_response(response: Dict[str, Any]) -> BackendResponse[Dict[str, Any]]:
@@ -333,3 +328,33 @@ class LambdaClient(Client):
             data=payload,
             status_code=status_code,
         )
+
+    def supports_async_uploads(self) -> bool:
+        return False
+
+    def supports_bulk_validate(self) -> bool:
+        return False
+
+    def supports_perf_test(self) -> bool:
+        return False
+
+    def get_metrics(self, params: MetricsParams) -> BackendResponse[MetricsResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("get metrics is not supported with lambda client")
+
+    def run_perf_test(self, params: PerfTestParams) -> BackendResponse[ReplayResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("run perf test is not supported with lambda client")
+
+    def supports_enrich_test_data(self) -> bool:
+        return False
+
+    def generate_enriched_event_input(
+        self, params: GenerateEnrichedEventParams
+    ) -> BackendResponse[GenerateEnrichedEventResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("enrich-test-data is not supported with lambda client")
+
+    def feature_flags(self, params: FeatureFlagsParams) -> BackendResponse[FeatureFlagsResponse]:
+        # pylint: disable=broad-exception-raised
+        raise BaseException("feature-flags is not supported with lambda client")
