@@ -16,6 +16,7 @@ GNU Affero General Public License for more details.
 You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
+
 import fnmatch
 import logging
 import os
@@ -32,8 +33,8 @@ from panther_analysis_tool.backend.client import BackendError, BackendResponse
 from panther_analysis_tool.backend.client import Client as BackendClient
 from panther_analysis_tool.backend.client import (
     ListSchemasParams,
-    ManagedSchema,
-    UpdateManagedSchemaParams,
+    Schema,
+    UpdateSchemaParams,
 )
 
 logger = logging.getLogger(__file__)
@@ -72,7 +73,7 @@ class Uploader:
     def __init__(self, path: str, backend: BackendClient):
         self._path = path
         self._files: Optional[List[str]] = None
-        self._existing_schemas: Optional[List[ManagedSchema]] = None
+        self._existing_schemas: Optional[List[Schema]] = None
         self._backend = backend
 
     @property
@@ -88,7 +89,7 @@ class Uploader:
         return self._files
 
     @property
-    def existing_schemas(self) -> List[ManagedSchema]:
+    def existing_schemas(self) -> List[Schema]:
         """
         Retrieves and caches in the instance state the list
         of available user-defined schemas.
@@ -97,13 +98,13 @@ class Uploader:
              List of user-defined schema records.
         """
         if self._existing_schemas is None:
-            resp = self._backend.list_managed_schemas(ListSchemasParams(is_managed=False))
+            resp = self._backend.list_schemas(ListSchemasParams(is_managed=False))
             if not resp.status_code == 200:
                 raise RuntimeError("unable to retrieve custom schemas")
             self._existing_schemas = resp.data.schemas
         return self._existing_schemas
 
-    def find_schema(self, name: str) -> Optional[ManagedSchema]:
+    def find_schema(self, name: str) -> Optional[Schema]:
         """
         Find schema by name.
 
@@ -111,7 +112,7 @@ class Uploader:
              The decoded YAML schema or None if no matching name is found.
         """
         for schema in self.existing_schemas:
-            if schema.name == name:
+            if schema.name.casefold() == name.casefold():
                 return schema
         return None
 
@@ -178,7 +179,7 @@ class Uploader:
             processed_file = ProcessedFile()
             processed_files[filename] = processed_file
             try:
-                with open(filename, "r") as schema_file:
+                with open(filename, "r", encoding="utf-8") as schema_file:
                     processed_file.raw = schema_file.read()
                 processed_file.yaml = yaml_parser.load(processed_file.raw)
             except (ParserError, ScannerError, ComposerError) as exc:
@@ -203,6 +204,9 @@ class Uploader:
                 f" with the prefix '{self._SCHEMA_NAME_PREFIX}'",
             )
 
+        if len(name) > 255:
+            return ("", "'schema' field: value should not exceed 255 characters")
+
         return name, None
 
     def _update_or_create_schema(
@@ -221,20 +225,26 @@ class Uploader:
             current_revision = existing_schema.revision
         reference_url = definition.get("referenceURL", current_reference_url)
         description = definition.get("description", current_description)
+        field_discovery_enabled = definition.get("fieldDiscoveryEnabled", False)
         logger.debug(
-            "updating schema %s at revision %d, using " "referenceURL=%s, " "description=%s",
+            "updating schema '%s' at revision '%d', using "
+            "referenceURL=%s, "
+            "description=%s, "
+            "fieldDiscoveryEnabled=%s",
             name,
             current_revision,
             reference_url,
             description,
+            field_discovery_enabled,
         )
-        response = self._backend.update_managed_schema(
-            params=UpdateManagedSchemaParams(
+        response = self._backend.update_schema(
+            params=UpdateSchemaParams(
                 name=name,
                 spec=processed_file.raw,
                 revision=current_revision,
                 reference_url=reference_url,
                 description=description,
+                field_discovery_enabled=field_discovery_enabled,
             )
         )
         return existed, response
@@ -292,7 +302,7 @@ def _contains_schema_tests(filename: str) -> bool:
 
     yaml_parser = YAML(typ="safe")
 
-    with open(filename, "r") as stream:
+    with open(filename, "r", encoding="utf-8") as stream:
         try:
             yaml_documents: List[Dict[str, Any]] = yaml_parser.load_all(stream)
         except (ParserError, ScannerError, ComposerError):
