@@ -5,6 +5,7 @@ from fnmatch import fnmatch
 from typing import Any, Dict, Generator, List, Optional, Set
 
 from panther_analysis_tool.analysis_utils import (
+    ClassifiedAnalysis,
     filter_analysis,
     load_analysis_specs,
     to_relative_path,
@@ -54,6 +55,7 @@ class ZipArgs:
     def from_args(cls, args: argparse.Namespace) -> Any:
         filters = []
         filters_inverted = {}
+        out = "./"
         try:
             filters = args.filter
         except:  # pylint: disable=bare-except # nosec
@@ -63,8 +65,13 @@ class ZipArgs:
             filters_inverted = args.filter_inverted
         except:  # pylint: disable=bare-except # nosec
             pass
+
+        try:
+            out = args.out
+        except:  # pylint: disable=bare-except # nosec
+            pass
         return cls(
-            out=args.out,
+            out=out,
             path=args.path,
             ignore_files=args.ignore_files,
             filters=filters,  # type: ignore
@@ -106,7 +113,7 @@ class ChunkFiles:
             return False
         return len(self.added_files) > self.chunk.max_size
 
-    def matches_file(self, filename: str, spec: Dict[str, Any] = None) -> bool:
+    def matches_file(self, filename: str, spec: Optional[Dict[str, Any]] = None) -> bool:
         if len(self.chunk.types) > 0 and spec is not None and "AnalysisType" in spec:
             if spec["AnalysisType"] not in self.chunk.types:
                 return False
@@ -138,7 +145,7 @@ def create_additional_chunks_if_needed(chunk_files: List[ChunkFiles]) -> List[Ch
     return results
 
 
-def analysis_chunks(args: ZipArgs, chunks: List[ZipChunk] = None) -> List[ChunkFiles]:
+def analysis_chunks(args: ZipArgs, chunks: Optional[List[ZipChunk]] = None) -> List[ChunkFiles]:
     """Generates all files that should be added to a zip file. If no chunks are provided
     a single chunk will be returned. Note: a file can be in multiple chunks if both chunks
     matches the file pattern
@@ -146,29 +153,40 @@ def analysis_chunks(args: ZipArgs, chunks: List[ZipChunk] = None) -> List[ChunkF
     :param chunks:
     :return:
     """
+    analysis = analysis_for_chunks(args)
+    return chunk_analysis(analysis, chunks)
 
-    if chunks is None or len(chunks) == 0:
-        chunks = [ZipChunk(patterns=["*"])]
 
-    chunk_files = [ChunkFiles(f) for f in chunks]
+def analysis_for_chunks(args: ZipArgs, no_helpers: bool = False) -> List[ClassifiedAnalysis]:
     analysis = []
     files: Set[str] = set()
 
-    for (file_name, f_path, spec, _) in list(
-        load_analysis_specs([args.path, HELPERS_LOCATION, DATA_MODEL_LOCATION], args.ignore_files)
-    ):
+    paths = [args.path]
+    if not no_helpers:
+        paths.extend([HELPERS_LOCATION, DATA_MODEL_LOCATION])
+    for file_name, f_path, spec, _ in list(load_analysis_specs(paths, args.ignore_files)):
         if file_name not in files:
-            analysis.append((file_name, f_path, spec))
+            analysis.append(ClassifiedAnalysis(file_name, f_path, spec))
             files.add(file_name)
             files.add("./" + file_name)
-    analysis = filter_analysis(analysis, args.filters, args.filters_inverted)
-    for analysis_spec_filename, dir_name, analysis_spec in analysis:
+    return filter_analysis(analysis, args.filters, args.filters_inverted)
+
+
+def chunk_analysis(
+    analysis: List[ClassifiedAnalysis], chunks: Optional[List[ZipChunk]] = None
+) -> List[ChunkFiles]:
+    if chunks is None or len(chunks) == 0:
+        chunks = [ZipChunk(patterns=["*"])]
+    chunk_files = [ChunkFiles(f) for f in chunks]
+    for item in analysis:
+        analysis_spec_filename = item.file_name
+        dir_name = item.dir_name
+        analysis_spec = item.analysis_spec
         main_file = to_relative_path(analysis_spec_filename)
         for chunk in chunk_files:
             if chunk.matches_file(analysis_spec_filename, analysis_spec):
                 chunk.add_file(main_file)
-                # datamodels may not have python body
-                if "Filename" in analysis_spec:
+                if analysis_spec is not None and "Filename" in analysis_spec:
                     chunk.add_file(
                         to_relative_path(os.path.join(dir_name, analysis_spec["Filename"])),
                         parent=main_file,
