@@ -17,79 +17,108 @@ You should have received a copy of the GNU Affero General Public License
 along with this program.  If not, see <http://www.gnu.org/licenses/>.
 """
 
-
 import unittest
 
 from panther_core.detection import DetectionResult
 from panther_core.policy import TYPE_POLICY
-from panther_core.rule import Rule, TYPE_RULE
+from panther_core.rule import TYPE_RULE, Rule
 
-from panther_analysis_tool.testing import FunctionTestResult, TestError, TestSpecification, \
-    TestExpectations, TestCaseEvaluator, TestResult, TestResultsPerFunction
+from panther_analysis_tool.testing import (
+    FunctionTestResult,
+    TestCaseEvaluator,
+    TestError,
+    TestExpectations,
+    TestResult,
+    TestResultsPerFunction,
+    TestSpecification,
+)
 
 TEST_RULE = {
-    'body': 'def rule(_):\n\treturn True',
-    'id': 'test-id',
-    'severity': 'INFO',
-    'type': TYPE_RULE,
-    'versionId': 'my-version',
+    "body": "def rule(_):\n\treturn True",
+    "id": "test-id",
+    "severity": "INFO",
+    "type": TYPE_RULE,
+    "versionId": "my-version",
 }
 
-class TestFunctionTestResult(unittest.TestCase):
 
+class TestFunctionTestResult(unittest.TestCase):
     def test_new(self) -> None:
         # If output is boolean
         result = FunctionTestResult.new(matched=True, output=True)
-        self.assertEqual(result, FunctionTestResult(matched=True, output='true', error=None))
+        self.assertEqual(result, FunctionTestResult(matched=True, output="true", error=None))
 
         # If output is string
-        result = FunctionTestResult.new(matched=True, output='some output')
-        self.assertEqual(result, FunctionTestResult(matched=True, output='some output', error=None))
+        result = FunctionTestResult.new(matched=True, output="some output")
+        self.assertEqual(result, FunctionTestResult(matched=True, output="some output", error=None))
 
         # If both parameters are None
         result = FunctionTestResult.new(matched=True, output=None, raw_exception=None)
         self.assertIsNone(result)
 
         # When an exception is given
-        exception = TypeError('wrong type')
-        result = FunctionTestResult.new(matched=False, output='some output', raw_exception=exception)
-        expected = FunctionTestResult(matched=False, output='some output', error=TestError(message='TypeError: wrong type'))
+        exception = TypeError("wrong type")
+        result = FunctionTestResult.new(
+            matched=False, output="some output", raw_exception=exception
+        )
+        expected = FunctionTestResult(
+            matched=False, output="some output", error=TestError(message="TypeError: wrong type")
+        )
         self.assertEqual(result, expected)
 
     def test_format_exception(self) -> None:
         self.assertIsNone(FunctionTestResult.format_exception(exc=None))
 
         # If title is None
-        exception = TypeError('wrong type')
-        self.assertEqual(FunctionTestResult.format_exception(exc=exception), 'TypeError: wrong type')
-        self.assertEqual(FunctionTestResult.format_exception(exc=exception, title='invalid type'), 'invalid type: TypeError: wrong type')
+        exception = TypeError("wrong type")
+        self.assertEqual(
+            FunctionTestResult.format_exception(exc=exception), "TypeError: wrong type"
+        )
+        self.assertEqual(
+            FunctionTestResult.format_exception(exc=exception, title="invalid type"),
+            "invalid type: TypeError: wrong type",
+        )
 
     def test_to_test_error(self) -> None:
         self.assertIsNone(FunctionTestResult.to_test_error(exc=None))
-        exception = TypeError('wrong type')
-        self.assertEqual(FunctionTestResult.to_test_error(exc=exception), TestError(message='TypeError: wrong type'))
+        exception = TypeError("wrong type")
         self.assertEqual(
-            FunctionTestResult.to_test_error(exc=exception, **dict(title='invalid type')),
-            TestError(message='invalid type: TypeError: wrong type')
+            FunctionTestResult.to_test_error(exc=exception),
+            TestError(message="TypeError: wrong type"),
+        )
+        self.assertEqual(
+            FunctionTestResult.to_test_error(exc=exception, **dict(title="invalid type")),
+            TestError(message="invalid type: TypeError: wrong type"),
         )
 
 
 class TestTestCaseEvaluator(unittest.TestCase):
-
     def test_interpret_passing_test_not_expected_to_trigger_alert(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=False))
-        detection_result = DetectionResult(detection_id=spec.id, trigger_alert=False, detection_output=False, detection_severity='INFO', detection_type=TYPE_RULE)
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=False),
+        )
+        detection_result = DetectionResult(
+            detection_id=spec.id,
+            trigger_alert=False,
+            detection_output=False,
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
+        )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=False,
             passed=True,
             trigger_alert=False,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output='false', error=None, matched=True),
+                detectionFunction=FunctionTestResult(output="false", error=None, matched=True),
                 titleFunction=None,
                 dedupFunction=None,
                 alertContextFunction=None,
@@ -97,26 +126,38 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
     def test_interpret_passing_test_expected_to_trigger_alert(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=True))
-        detection_result = DetectionResult(detection_id=spec.id, trigger_alert=True, detection_output=True, detection_severity='INFO', detection_type=TYPE_RULE)
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=True),
+        )
+        detection_result = DetectionResult(
+            detection_id=spec.id,
+            trigger_alert=True,
+            detection_output=True,
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
+        )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=False,
             passed=True,
             trigger_alert=True,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output='true', error=None, matched=True),
+                detectionFunction=FunctionTestResult(output="true", error=None, matched=True),
                 titleFunction=None,
                 dedupFunction=None,
                 alertContextFunction=None,
@@ -124,32 +165,40 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(actual, expected)
 
     def test_interpret_failing_test_expected_to_trigger_alert(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=True))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=True),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
-            detection_exception=TypeError('wrong type'),
-            detection_severity='INFO',
-            detection_type=TYPE_RULE
+            detection_exception=TypeError("wrong type"),
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=True,
             passed=False,
             trigger_alert=False,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output=None, error=TestError(message='TypeError: wrong type'), matched=False),
+                detectionFunction=FunctionTestResult(
+                    output=None, error=TestError(message="TypeError: wrong type"), matched=False
+                ),
                 titleFunction=None,
                 dedupFunction=None,
                 alertContextFunction=None,
@@ -157,66 +206,82 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
     def test_interpret_failing_test_expected_to_match_aux_function_error(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=True))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=True),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=True,
             detection_output=True,
-            detection_severity='INFO',
+            detection_severity="INFO",
             detection_type=TYPE_RULE,
-            title_exception=TypeError('wrong type')
+            title_exception=TypeError("wrong type"),
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=True,
             passed=False,
             trigger_alert=True,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output='true', error=None, matched=True),
-                titleFunction=FunctionTestResult(output=None, error=TestError(message='TypeError: wrong type'), matched=False),
+                detectionFunction=FunctionTestResult(output="true", error=None, matched=True),
+                titleFunction=FunctionTestResult(
+                    output=None, error=TestError(message="TypeError: wrong type"), matched=False
+                ),
                 dedupFunction=None,
                 alertContextFunction=None,
                 descriptionFunction=None,
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
     def test_interpret_failing_test_not_expected_to_trigger_alert_detection_error(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=False))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=False),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
-            detection_exception=TypeError('wrong type'),
-            detection_severity='INFO',
-            detection_type=TYPE_RULE
+            detection_exception=TypeError("wrong type"),
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=True,
             passed=False,
             trigger_alert=False,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output=None, error=TestError(message='TypeError: wrong type'), matched=False),
+                detectionFunction=FunctionTestResult(
+                    output=None, error=TestError(message="TypeError: wrong type"), matched=False
+                ),
                 titleFunction=None,
                 dedupFunction=None,
                 alertContextFunction=None,
@@ -224,38 +289,44 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
     def test_interpret_failing_test_not_expected_to_trigger_alert_with_aux_exception(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=False))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=False),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
             detection_output=False,
             detection_exception=None,
-            detection_severity='INFO',
+            detection_severity="INFO",
             detection_type=TYPE_RULE,
-            destinations_exception=TypeError('wrong type'),
-            reference_exception=TypeError('wrong type'),
-            runbook_exception=TypeError('wrong type'),
-            severity_exception=TypeError('wrong type'),
-            title_exception=TypeError('wrong type'),
+            destinations_exception=TypeError("wrong type"),
+            reference_exception=TypeError("wrong type"),
+            runbook_exception=TypeError("wrong type"),
+            severity_exception=TypeError("wrong type"),
+            title_exception=TypeError("wrong type"),
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=True,
             passed=True,
             trigger_alert=False,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output='false', error=None, matched=True),
+                detectionFunction=FunctionTestResult(output="false", error=None, matched=True),
                 titleFunction=None,
                 dedupFunction=None,
                 alertContextFunction=None,
@@ -264,37 +335,45 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 severityFunction=None,
                 runbookFunction=None,
                 destinationsFunction=None,
-            )
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
-    def test_interpret_failing_test_policy_not_expected_to_trigger_alert_with_aux_exception(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=True))
+    def test_interpret_failing_test_policy_not_expected_to_trigger_alert_with_aux_exception(
+        self,
+    ) -> None:
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=True),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
-            detection_output=True, # policys return true when not triggering an alert
+            detection_output=True,  # policys return true when not triggering an alert
             detection_exception=None,
-            detection_severity='INFO',
+            detection_severity="INFO",
             detection_type=TYPE_POLICY,
-            destinations_exception=TypeError('wrong type'),
-            reference_exception=TypeError('wrong type'),
-            runbook_exception=TypeError('wrong type'),
-            severity_exception=TypeError('wrong type'),
-            title_exception=TypeError('wrong type'),
+            destinations_exception=TypeError("wrong type"),
+            reference_exception=TypeError("wrong type"),
+            runbook_exception=TypeError("wrong type"),
+            severity_exception=TypeError("wrong type"),
+            title_exception=TypeError("wrong type"),
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
             genericError=None,
             error=None,
             errored=True,
             passed=True,
             trigger_alert=False,
             functions=TestResultsPerFunction(
-                detectionFunction=FunctionTestResult(output='true', error=None, matched=True),
+                detectionFunction=FunctionTestResult(output="true", error=None, matched=True),
                 titleFunction=None,
                 dedupFunction=None,
                 alertContextFunction=None,
@@ -303,26 +382,32 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 severityFunction=None,
                 runbookFunction=None,
                 destinationsFunction=None,
-            )
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
     def test_interpret_failing_test_input_error(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=False))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=False),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
-            input_exception=TypeError('wrong type'),
-            detection_severity='INFO',
-            detection_type=TYPE_RULE
+            input_exception=TypeError("wrong type"),
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
-            genericError='Invalid event: TypeError: wrong type',
-            error=TestError(message='Invalid event: TypeError: wrong type'),
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
+            genericError="Invalid event: TypeError: wrong type",
+            error=TestError(message="Invalid event: TypeError: wrong type"),
             errored=True,
             passed=False,
             trigger_alert=False,
@@ -335,27 +420,33 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
     def test_interpret_generic_error(self) -> None:
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=False))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=False),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
-            setup_exception=TypeError('wrong type'),
-            detection_severity='INFO',
-            detection_type=TYPE_RULE
+            setup_exception=TypeError("wrong type"),
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
-            genericError='TypeError: wrong type',
-            error=TestError(message='TypeError: wrong type'),
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
+            genericError="TypeError: wrong type",
+            error=TestError(message="TypeError: wrong type"),
             errored=True,
             passed=False,
             trigger_alert=False,
@@ -368,27 +459,33 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
 
         # Event compatibility exception
-        spec = TestSpecification(id='test-id', name='test-name', data={}, mocks=[], expectations=TestExpectations(detection=False))
+        spec = TestSpecification(
+            id="test-id",
+            name="test-name",
+            data={},
+            mocks=[],
+            expectations=TestExpectations(detection=False),
+        )
         detection_result = DetectionResult(
             detection_id=spec.id,
             trigger_alert=False,
-            input_exception=TypeError('wrong type'),
-            detection_severity='INFO',
-            detection_type=TYPE_RULE
+            input_exception=TypeError("wrong type"),
+            detection_severity="INFO",
+            detection_type=TYPE_RULE,
         )
         expected = TestResult(
-            id='test-id',
-            name='test-name',
-            detectionId='test-id',
-            genericError='Invalid event: TypeError: wrong type',
-            error=TestError(message='Invalid event: TypeError: wrong type'),
+            id="test-id",
+            name="test-name",
+            detectionId="test-id",
+            genericError="Invalid event: TypeError: wrong type",
+            error=TestError(message="Invalid event: TypeError: wrong type"),
             errored=True,
             passed=False,
             trigger_alert=False,
@@ -401,8 +498,8 @@ class TestTestCaseEvaluator(unittest.TestCase):
                 referenceFunction=None,
                 severityFunction=None,
                 runbookFunction=None,
-                destinationsFunction=None
-            )
+                destinationsFunction=None,
+            ),
         )
         actual = TestCaseEvaluator(spec=spec, detection_result=detection_result).interpret()
         self.assertEqual(expected, actual)
